#' @title ava
#'
#' @description
#' Core function of the AVA model. Vulnerability assessment at
#' the farm scale.
#'
#' @param P list of parameters obtained from `parameter()`.
#' @param testing boolean, indicated whether it is a test or not, default to FALSE.
#' @param verbose boolean, indicates whether to export state of components in 
#' `flood_functioning()`, default to TRUE.
#' @param s character, name of hazard scenario to process when `testing = TRUE`, default to NULL
#'
#' @return TO BE PRECISED
#'
#' @export
#'
#' @encoding UTF-8
 
ava = function(
    P,
    testing = FALSE,
    verbose = TRUE,
    s = NULL
) {
    r = list()

    ################################################################################
    ### 1. Loading Parameters
    ################################################################################
    if (testing) print("##### Loading Parameters #################")
    r[["Parameters"]] = farm_parameters(P, testing)
    if (testing) print("##### Loading Parameters: OK #############")
    
    ################################################################################
    ### 2. Normal functionning
    ################################################################################
    if (testing) print("##### Normal Functioning ################")
    r[["Normal"]] = normal_functioning(r[["Parameters"]], testing)
    if (testing) print("##### Normal Functionning: OK ############")

    ################################################################################
    ### 3. Flooded functionning
    ################################################################################

    if (testing)
    {
        if (testing) print("##### Flooded Functioning ###############")
        if (is.null(s)) s = names(r[["Parameters"]][["Hazard"]])[1]
        r[[paste("f", s, sep = "")]] = flood_functioning(
            P = r[["Parameters"]], 
            S = r[["Parameters"]][["Hazard"]][[s]], 
            N = r[["Normal"]],
            verbose = verbose,
            testing = testing)
        print("##### Flooded Functioning: OK ###########")
        print(analyse_income(r))
        print(analyse_damage(r))
        print(r[[paste("f", s, sep = "")]][["Damage"]][["Financial"]])
        return(r)
    }

    for (s in names(r[["Parameters"]][["Hazard"]]))
    {
        r[[paste("f", s, sep = "")]] = flood_functioning(
            P = r[["Parameters"]], 
            S = r[["Parameters"]][["Hazard"]][[s]], 
            N = r[["Normal"]],
            verbose = verbose)
    }
    
    Income = analyse_income(r)
    Damage = analyse_damage(r)
    
    r[["Income"]] = Income
    r[["Damage"]] = Damage
    
    invisible(r)
}
