#' @title calendar_input
#'
#' @description
#' TO BE PRECISED
#'
#' @param CC matrix, calendar of input consumption and their amounts. Result of `need_resource`
#' @param purchase data.frame, input purchase. Result of `load_parameter(P[["Input_Purchase"]],...)`
#' @param path character, path of farm data
#' @param profile character, profile type.
#' @param p TO BE PRECISED, purchase. Default to NULL.
#' @param cal_length numeric, number of weeks in calendar. Default to
#' `getOption("ava")[["cal_length"]]`.
#' 
#' @return matrix, evolution of input stocks for each week of the year.
#'
#' @export
#'
#' @encoding UTF-8

calendar_input = function(
    CC,
    purchase,
    path,
    profile,
    p = NULL,
    cal_length = getOption("ava")[["cal_length"]]
) {
    if (!(all(rownames(CC) %in% unique(purchase[, "input"]))))
        stop("rownames of consumption not compatible with rownames of purchase")

    # Some verifications
    input_consumed = apply(CC, 1, sum)
    temp = stats::aggregate(
        purchase[, "quantity"],
        by = list("input" = purchase[, "input"]),
        sum
    )
    input_purchased = temp[["x"]]
    names(input_purchased) = temp[["input"]]
    input_purchased = input_purchased[names(input_consumed)]
    if (any(test <- input_consumed > input_purchased)) {
        msg =
            paste(
                path,
                profile,
                ": some input (",
                paste(
                    names(input_consumed)[test], 
                    collapse = ", "
                ), ") are more consumed than purchased...\n",
                sep = ""
            )
        msg = paste(msg, "\tinput:consumed(purchased)\n")
        msg = paste(
            msg,
            paste(
                "\t",
                names(input_consumed)[test],
                ":",
                input_consumed[test],
                "(",
                input_purchased[test],
                ")",
                sep = "",
                collapse = "\n"
            ),
            sep = ""
        )
        warning(msg)
    }

    CC = make_coherent(CC, rows = unique(purchase[, "input"]))
    A = stats::aggregate(
        t(apply(purchase[, c("week","quantity")], 1, calendarize)), 
        by = list("input" = purchase[, "input"]),
        sum
    )
    rownames(A) = A[, "input"]
    Flux = as.matrix(A[rownames(CC), -1]) - CC
    b = purchase[purchase[, "start"], ]
    b = b[match(b[, "input"], rownames(CC)), "week"]
    r = apply(cbind(b, Flux), 1, flux_to_stock)
    if (!is.null(p)) {
        return(r[(seq(ncol(r)) - 1) * cal_length + p])
    }
    r = round(t(r), 2)
    return(r)
}