#' @title damaging
#'
#' @description
#' DESCRIPTION NEEDED
#'
#' @param Damaging array with named dimensions. All named dimensions except
#' optional "output" refer to hazard dimensions. Default to NULL.
#' @param Hazard matrix with named columns corresponding to hazard dimensions.
#' Default to NULL.
#' @param method character, method used to create damaging. Use "exact" when you
#' are sure that points in Hazard are expressed as levels used in Damaging.
#' Default to "constant".
#' @param output optional parameter giving vector of desired outputs that
#' shall be used if Damaging has "output" in its named dimensions. Default to
#' NULL.
#' @param day numeric, number of days in a week. Default to
#' `getOption("ava")[["day"]]`.
#' @param day_offset numeric, number of days of offset due to waterlogging. 
#' Default to `getOption("ava")[["day_offset"]]`.
#'
#' @return TO BE PRECISED
#'
#' @export
#'
#' @encoding UTF-8

damaging = function(
    Damaging = NULL,
    Hazard = NULL,
    method = "constant",
    output = NULL,
    day = getOption("ava")[["day"]],
    day_offset = getOption("ava")[["day_offset"]]
) {
    if (is.null(Damaging) | is.null(Hazard)) return()
    if (any(is.na(pmatch(method, c("exact", "linear", "constant", duplicates.ok = TRUE)))))
        stop("Invalid method.")
    if (all(is.na(pmatch(method, c("linear", "constant", duplicates.ok = TRUE)))))
        method = "exact" else
        method[method == "exact"] = "constant"
    if (is.vector(Hazard)) 
        Hazard = t(Hazard)
    if ("duration" %in% colnames(Hazard))
        Hazard[, "duration"] = (Hazard[, "duration"] - 1) * day + day_offset
    
    if ("output" %in% names(dimnames(Damaging)))
    {
        hazard = names(dimnames(Damaging))[!(names(dimnames(Damaging)) %in% "output")]
        o = which(names(dimnames(Damaging)) == "output")
        result = apply(Damaging, o, damaging, 
            Hazard = Hazard[,hazard], method = method)
        if (is.vector(result)) result = t(result)
        if (!is.null(output)) 
            result = result[, pmatch(output, dimnames(Damaging)[["output"]], duplicates.ok = TRUE)] ### CHECK!!!
        return(result)
    } else {
        hazard = names(dimnames(Damaging))       
        Hazard = Hazard[, hazard]
        if (is.vector(Hazard)) Hazard = t(Hazard)
        
        if (method == "exact")
        {
            cell = 1 + apply((Hazard-1) * 
                cumprod(c(1, dim(Damaging)[-length(dim(Damaging))])), 1, sum)
            result = Damaging[cell]
            names(result) = names(cell)
        } else {
            cycling = names(dimnames(Damaging)) %in% "week"
            result = interpolate(Hazard, Damaging, method = method, cycling = cycling)
        }
        return(result)
    }
}