#' @title farm_parameters
#'
#' @description
#' Load farm parameters.
#'
#' @param P list of parameters file names, result of parameter().
#' @param testing boolean, indicates whether it is a test or not, default to FALSE
#'
#' @return list of farm parameters to be passed in normal_functioning() and flood_functioning()
#'
#' @export
#'
#' @encoding UTF-8

farm_parameters = function(P, testing = FALSE) {
    r = list()
    
    r[["signature"]] = P
    r[["path"]] = P[["path"]]

    if (testing) print("Loading Labour...")
        r[["Labour"]] = load_parameter(P[["Labour"]], r[["path"]])
    if (testing) print("...OK")

    if (testing) print("Loading Farm_Labour...")
        r[["Farm_Labour"]] = load_parameter(P[["Farm_Labour"]], r[["path"]])    
        r[["labour_i"]] = r[["Farm_Labour"]]["internal_hour", ] * 
            r[["Farm_Labour"]]["internal", ]
    if (testing) print("...OK")

    if (testing) print("Loading Farm_Spatial...")
        r[["Farm_Spatial"]] = load_parameter(P[["Farm_Spatial"]], r[["path"]])
        r[["l_plot"]] = rownames(
            r[["Farm_Spatial"]][r[["Farm_Spatial"]]["class"] == "plot", ]
        )
        r[["l_building"]] = rownames(r[["Farm_Spatial"]][
            r[["Farm_Spatial"]]["class"] == "building", ])

        crop = levels(unique(r[["Farm_Spatial"]][, "crop"]))
        building = c("building")["building" %in% r[["Farm_Spatial"]][, "class"]]
    if (testing) print("...OK")

    if (testing) print("Loading Furniture...")
        r[["Furniture"]] = load_parameter(P[["Furniture"]], r[["path"]])
        r[["tool"]] = rownames(
            r[["Furniture"]]
        )[r[["Furniture"]][, "class"] == "tool"]
        r[["input"]] = rownames(
            r[["Furniture"]]
        )[r[["Furniture"]][, "class"] == "input"]
    if (testing) print("...OK")

    if (testing) print("Loading Input...")
        r[["Input"]] = load_parameter(P[["Input"]], r[["path"]])
    #~     r[["Input"]] = load_parameter(
    #~          P[["Input"]],
    #~          r[["path"]]
    #~     )[r[["input"]], "price"]
    #~     r[["Input"]] = data.frame(matrix(r[["Input"]], ncol = 1, 
    #~         dim = list("input" = r[["input"]], "parameter" = "price")))
    if (testing) print("...OK")

    if (testing) print("Loading Input_Purchase...")
        r[["Input_Purchase"]] = load_parameter(
            P[["Input_Purchase"]],
            r[["path"]],
            row.names = NULL
        )
    if (testing) print("...OK")

    if (testing) print("Loading Tool...")
        r[["Tool"]] = load_parameter(P[["Tool"]], r[["path"]])
    #~     r[["Tool"]] = load_parameter(P[["Tool"]], r[["path"]])[r[["tool"]], ]
    if (testing) print("...OK")

    if (testing) print("Loading CMS...")
        r[["CMS"]] = load_by_plot(
            "CMS",
            r[["Farm_Spatial"]][r[["l_plot"]], ],
            r[["path"]]
        )
        r[["CMS"]][is.na(r[["CMS"]][, "loss"]), "loss"] = 0
    if (testing) print("...OK")

    if (testing) print("Loading Task_Tool...")
        r[["Task_Tool"]] = load_by_plot(
            "Task_Tool",
            r[["Farm_Spatial"]][r[["l_plot"]], ],
            r[["path"]]
        )
    if (testing) print("...OK")

    if (testing) print("Loading Task_Input...")
        r[["Task_Input"]] = load_by_plot(
            "Task_Input",
            r[["Farm_Spatial"]][r[["l_plot"]], ],
            r[["path"]]
        )
        r[["Task_Input"]] = r[["Task_Input"]] / r[["CMS"]][, "labour"]
    if (testing) print("...OK")

    if (testing) print("Loading Task_Flood...")
        r[["Task_Flood"]] = select_task(
            load_parameter(P[["Task_Flood"]], r[["path"]]), 
            c(crop, building))
    if (testing) print("...OK")

    if (testing) print("Loading Task_Flood_Input...")
        r[["Task_Flood_Input"]] = load_task_resource(
            "Task_Flood_Input.csv",
            r[["path"]],
            rownames(r[["Task_Flood"]]),
            colnames(r[["Task_Input"]]),
            r[["Farm_Spatial"]]
        )
    if (testing) print("...OK")

    if (testing) print("Loading Task_Flood_Tool...")
        r[["Task_Flood_Tool"]] = load_task_resource(
            "Task_Flood_Tool.csv", 
            r[["path"]],
            rownames(r[["Task_Flood"]]),
            colnames(r[["Task_Tool"]]),
            r[["Farm_Spatial"]]
        )
    if (testing) print("...OK")

    if (testing) print("Loading Hazard...")
        r[["Hazard"]] = prepare_hazard(
            load_parameter(P[["Hazard"]], r[["path"]], row.names = NULL)
        )
    if (testing) print("...OK")

    if (testing) print("Loading Damaging_Building...")
        r[["Damaging_Building"]] = make_damaging(
            load_parameter(
                P[["Damaging_Building"]],
                r[["path"]],
                row.names = NULL
            )
        )
    if (testing) print("...OK")

    if (testing) print("Loading Damaging_Furniture...")
        r[["Damaging_Furniture"]] = load_parameter(
            P[["Damaging_Furniture"]],
            r[["path"]],
            row.names = NULL
        )
        r[["Type_Furniture"]] = unique(
            r[["Damaging_Furniture"]][c("vulnerability", "type")]
        )
        r[["Damaging_Furniture"]] = make_damaging(
            r[["Damaging_Furniture"]][, -1]
        )
    if (testing) print("...OK")

    if (testing) print("Loading Discount_Tool...")
        r[["Discount_Tool"]] = load_parameter(P[["Discount_Tool"]], r[["path"]], 
            row.names = NULL)
    if (testing) print("...OK")

    if (testing) print("Making Tool, Input, Table_Furniture...")
        r[["Tool"]][["type"]] = r[["Type_Furniture"]][match(
            r[["Tool"]][["vulnerability"]], 
            r[["Type_Furniture"]][["vulnerability"]]
        ), "type"]
        r[["Input"]][["type"]] = r[["Type_Furniture"]][match("input", 
            r[["Type_Furniture"]][["vulnerability"]]), "type"]
        r[["Table_Furniture"]] = rbind(r[["Tool"]][c("type", "price")], 
            r[["Input"]][c("type", "price")])
    if (testing) print("...OK")

        r[["Furniture"]] = cbind(
            r[["Furniture"]],
            r[["Table_Furniture"]][rownames(r[["Furniture"]]), ]
        )

    if (testing) print("Loading Damaging_Soil...")
        r[["Damaging_Soil"]] = load_damaging(
            "Damaging_Soil", r[["Farm_Spatial"]][r[["l_plot"]], ], r[["path"]]
        )
    if (testing) print("...OK")

    if (testing) print("Loading Damaging_Culture...")
        r[["Damaging_Culture"]] = load_damaging(
            "Damaging_Culture",
            r[["Farm_Spatial"]][r[["l_plot"]], ],
            r[["path"]]
        )
    if (testing) print("...OK")

    if (testing) print("Loading Damaging_PlantMaterial...")
        r[["Damaging_PlantMaterial"]] = load_damaging(
            "Damaging_PlantMaterial",
            r[["Farm_Spatial"]][r[["l_plot"]], ],
            r[["path"]]
        )
    if (testing) print("...OK")

    if (testing) print("Loading Cost_PlantMaterial...")
        r[["Cost_PlantMaterial"]] = load_parameter(
            "Cost_PlantMaterial.csv",
            r[["path"]]
        )
    if (testing) print("...OK")

    if (testing) print("Making Cost...")
        r[["Cost"]] = make_cost(
            r[["Labour"]],
            r[["Input"]],
            r[["Tool"]]
        )
    if (testing) print("...OK")
            
    if (testing) print("Loading External_Calendar...")
        r[["Calendar"]] = load_calendar(
            r[["tool"]],
            r[["input"]],
            r[["Furniture"]],
            r[["labour_i"]],
            r[["path"]]
        )
    if (testing) print("...OK")
        
    if (testing) print("Making profile, product...")
        r[["profile"]] = P[["profile"]]
        r[["product"]] = sum(
            apply(
                r[["Farm_Spatial"]][c("area", "yield", "price")],
                1,
                prod
            ),
            na.rm = TRUE
        )
    if (testing) print("...OK")

    if (testing)
    {
        test_names(
            r[["Calendar"]],
            r[["Cost"]][["Financial"]],
            "Calendar",
            'Cost[["Financial"]]', 
            paste(r[["path"]], r[["profile"]], sep = "")
        )
    }
        return(r)
}