#' @title flood_functioning
#'
#' @description
#' TO BE PRECISED.
#'
#' @param P TO BE PRECISED.
#' @param S TO BE PRECISED.
#' @param N TO BE PRECISED. Default value to NULL.
#' @param testing logical, TO BE PRECISED. Default value to FALSE.
#' @param verbose logical, TO BE PRECISED. Default value to FALSE.
#'
#' @return TO BE PRECISED
#'
#' @export
#'
#' @encoding UTF-8

flood_functioning = function(P, S, N = NULL, testing = FALSE, verbose = FALSE) {
    
    r = list()
    
    ### 1. Checking if normal functionning has been computed.
    if (is.null(N)) N = normal_functioning(P)
#~     Task_Input = rbind(P[["Task_Input"]], P[["Task_Flood_Input"]]) # 2011-07-28 OLD
#~     Task_Tool = rbind(P[["Task_Tool"]], P[["Task_Flood_Tool"]]) # 2011-07-28 OLD
    Task_Input = coherency(P[["Task_Input"]], P[["Task_Flood_Input"]]) # 2011-07-28 DEV
    Task_Tool = coherency(P[["Task_Tool"]], P[["Task_Flood_Tool"]]) # 2011-07-28 DEV
#~     print(Task_Tool) # 2011-07-28 DEV
    if (testing) print("1. OK")
    
    ### 2. Flooding
    
    ### 2.0 Adapting hazard ! 
    # Caution: for plot (only) the operation will be undone when calculating 
    #   damage to soil.
    s = rownames(S)
    S[s, "height"] = S[s,  "height"] - P[["Farm_Spatial"]][s, "elevation"]
    if (testing) print("2.0. OK")
    
    ### 2.1 Direct damage to Building
    Task_Building = task_building(
        S,
        P[["Farm_Spatial"]],
        P[["Damaging_Building"]],
        P[["Task_Flood"]]
    )
    if (testing) print("2.1. OK")
    
    ### 2.2 Direct damage to Furniture
    Furniture = hazard_furniture(P[["Furniture"]], S, P[["Damaging_Furniture"]], 
        P[["Discount_Tool"]], N[["Input"]])
    if (testing) print("2.2. OK")
    
    ### 2.3. Direct damaging in plot
    ### Direct Plot
    Task_Plot = task_plot(
        P[["Damaging_Soil"]],
        P[["Farm_Spatial"]],
        P[["Task_Flood"]],
        S, 
        P[["l_plot"]]
    )
    harvest = extract_harvest(P[["CMS"]], P[["l_plot"]])
    Loss_Plot = loss_plot(P[["Damaging_PlantMaterial"]], 
        P[["Damaging_Culture"]], S, P[["l_plot"]], harvest)
    loss_plantmaterial = estimate_dam_plantmaterial(Loss_Plot, P[["Farm_Spatial"]], 
        P[["Cost_PlantMaterial"]], P[["l_plot"]])
    if (testing) print("2.3. OK")
    
    ### 4. Adapting Tasks
    ### 4.1 Stop CMS if Yield_Loss_Direct > threshold
    CMS = adapt_cms(P[["CMS"]], Loss_Plot, P[["l_plot"]], S)
    #CMS = P[["CMS"]]
    if (testing) print("4.1. OK")
    
    ### 4.2 Adding Task_Building and Task_Flood
    CMS = rbind(CMS, Task_Building, Task_Plot)
    if (testing) print("4.2. OK")
    
    ### 4.3 Adapting tasks to flood
    e = rownames(P[["Farm_Spatial"]])
    Task = distribute_labour(CMS[, c("labour", "begin", "end")]) *
        fit_area(e, P[["Farm_Spatial"]][, "area"], rownames(CMS))
    Task = shift_labour(CMS[, "end"], Task, 
        fit_length(e, 
        cbind(S[e, "week"], fit_date(S[e, "week"] + S[e, "duration"] - 1)), 
        rownames(CMS)))
    if (testing) print("4.3. OK")

    ### 4.4 Availability of tools
    Tool = P[["Calendar"]][["tool"]]
    Tool[["internal"]][!usability(Furniture[P[["tool"]], c("week", "duration", 
        "unavailability")])] = 0
    if (testing) print("4.4. OK")
    
    ### 4.5 Availability of inputs ### NOT TAKING INTO ACCOUNT LOSS
    Input = list()
    Input[["internal"]] = need_resource(Task, Task_Input)
    Input[["external"]] = init_calendar(rownames(Input[["internal"]]))
    if (testing) print("4.5. OK")

    ### 4.6 Calendar of available labour
    Labour = P[["Calendar"]][["labour"]]
    # Applying crisis time to labour
	w.b = min(S[, "week"], na.rm = TRUE) + 1
	w.e = w.b + P[["Farm_Labour"]]["duration_crisis", "value"]
	if (w.b != Inf)
	{
		selection = select_calendar(fit_date(c(w.b, w.e)))
		Labour[["internal"]][selection] = 
			P[["Farm_Labour"]]["internal_hour_crisis", "value"] *
			P[["Farm_Labour"]]["internal", "value"] 
	}
    if (testing) print("4.6. OK")

    ### 5 Global resolution
    Calendar = solve_task(Task, Input, Tool, Labour, Task_Input, Task_Tool, 
        other = list(
            "building" = rownames(Task_Building),
            "soil" = rownames(Task_Plot)), 
        p = P[["profile"]])
    if (testing) print("5. OK")

    ### 6. Expenses
    selection = names(Calendar)[!("task" == names(Calendar))]
    Expense = list()
    Expense[["Financial"]] = lapply(
        Calendar[selection],
        estimate_expense,
        P[["Cost"]][["Financial"]]
    )
	# For internal profile only, calculating due costs, independantly to realization of task
    if (P[["signature"]][["profile"]] == "internal")
    {
		Expense[["Financial"]][["building"]]["tool"] = 
			estimate_expense_direct(Task_Building, P[["Task_Flood_Tool"]], 
			P[["Cost"]][["Financial"]][["tool"]], P[["Farm_Spatial"]])
		Expense[["Financial"]][["soil"]]["tool"] =
			estimate_expense_direct(Task_Plot, P[["Task_Flood_Tool"]], 
			P[["Cost"]][["Financial"]][["tool"]], P[["Farm_Spatial"]])
	}
	Expense[["Economic"]] = lapply(
        Calendar[selection], estimate_expense,  P[["Cost"]][["Economic"]]
    )
    Solidarity = solidarity(
        Calendar[selection], N[["Calendar"]],
        P[["Cost"]][["Financial"]],
        Expense[["Financial"]]
    )
    if (testing) print("6. OK")

    ### 7. Product
    Labour_Loss = CMS[, c("labour", "loss")]
    Labour_Loss[rownames(P[["CMS"]]), ] = P[["CMS"]][, c("labour", "loss")]
    Product = estimate_product(
        Calendar[["task"]],
        Labour_Loss, 
        P[["Farm_Spatial"]][
            P[["l_plot"]],
            c("yield", "area", "price"),
            drop = FALSE
        ], 
        Loss_Plot[, c("plant_material", "culture"), drop = FALSE]
    )
    if (testing) print("7. OK")
        
    ### 8. Incomes
    r[["Income"]] = estimate_income(Expense[["Financial"]][["plot"]], Product)
    if (testing) print("8. OK")
    
    ### 9. Damages
    r[["Damage"]] = list()
    r[["Damage"]][["Financial"]] = estimate_dam_financial(NP = N[["Product"]], NE = N[["Expense"]][["Financial"]],
        P = Product, E = Expense[["Financial"]], FU = Furniture,          
        culture = as.character(P[["Farm_Spatial"]][P[["l_plot"]], "crop"]), D = loss_plantmaterial)
    r[["Damage"]][["Solidarity"]] = estimate_dam_financial(NP = N[["Product"]], NE = N[["Expense"]][["Financial"]],
        P = Product, E = Solidarity[["Expense"]], FU = Furniture,          
        culture = as.character(P[["Farm_Spatial"]][P[["l_plot"]], "crop"]), D = loss_plantmaterial)
    r[["Damage"]][["Economic"]] = estimate_dam_economic(N[["Product"]], N[["Expense"]][["Economic"]], 
        Product, Expense[["Economic"]], Furniture, loss_plantmaterial)
#~     r[["Damage"]] = balance_damage(r[["Income"]], N[["Income"]], Furniture, Expense[["Financial"]], loss_plantmaterial)
    if (testing) print("9. OK")
    
    if (verbose)
    {
        r[["Calendar"]] = Calendar
        r[["CMS"]] = CMS
        r[["Expense"]] = Expense
        r[["Furniture"]] = Furniture
        r[["Labour"]] = Labour
        r[["Labour_Loss"]] = Labour_Loss
        r[["Loss_Plot"]] = Loss_Plot
        r[["Input"]] = Input
        r[["Product"]] = Product
        r[["S"]] = S
        r[["Solidarity"]] = Solidarity
        r[["Task"]] = Task
        r[["Task_Input"]] = Task_Input
        r[["Task_Tool"]] = Task_Tool
        r[["Task_Building"]] = Task_Building
        r[["Task_Plot"]] = Task_Plot
        r[["Tool"]] = Tool
        # State of components -> availability of tools.
    }

    return(r)
}
