#' @title load_parameter
#'
#' @description
#' Reading farm parameter from csv (semi-colon) file to the correct format.
#'
#' @param parameter character, name of the file of parameter to be loaded.
#' @param path character, subdirectory where to find parameters.
#' @param m logical, should the result be returned as a matrix.
#' @param generic character, default name for the file of parameters to be
#' loaded. Default to `getOption("ava")[["generic"]]`.
#' @param path_input character, directory where to find `path`. Default to
#' `getOption("ava")[["path_input"]]`.
#' @param row.names integer, columns where to find row.names. Default to 1.
#' @param checking logical, should the parameters be checked. Default to
#' `getOption("ava")[["checking"]]`.
#' @param ... other parameters that will be passed to `utils::read.csv2()`.
#'
#' @return Either a data.frame or a matrix of parameters.
#'
#' @export
#'
#' @encoding UTF-8

load_parameter = function(
    parameter,
    path,
    m = FALSE,
    generic = getOption("ava")[["generic"]], 
    path_input = getOption("ava")[["path_input"]],
    row.names = 1,
    checking = getOption("ava")[["checking"]],
    ...
) {
    path = file.path(path_input, path)
    if (!(parameter %in% dir(path)))
        path = file.path(path_input, generic)
    if (!(parameter %in% dir(path)))
        stop(paste("No file", parameter, "found..."))
    
    parameter = file.path(path, parameter)
    if (checking) check(parameter)
    r = utils::read.csv2(
        parameter,
        row.names = row.names,
        check.names = FALSE,
        stringsAsFactors = TRUE,
        ...
    )
    if (m) r = as.matrix(r)
    return(r)
}
