#' @title localize_hazard
#'
#' @description
#' TO BE PRECISED.
#'
#' @param elevation numeric, elevation of each asset.
#' @param hazard matrix or vector of parameter. If a vector, `hazard` is
#' transformed in a matrix. Default to NULL.
#' @param asset character, names of the assets. Default to NULL.
#' @param b_elevation elevation of the building in which asset are located.
#' May be already taken into account in hazard. Default to 0.
#' @param ordered_parameter character, hazard dimensions. Default to 
#' `getOption("ava")[["generic"]]`.
#'
#' @return matrix of hazard values, rows indicate assets, columns indicate
#' hazard scenarios.
#'
#' @export
#'
#' @encoding UTF-8

localize_hazard = function(
    elevation,
    hazard = NULL,
    asset = NULL,
    b_elevation = 0,
    ordered_parameter = getOption("ava")[["generic"]]
) {
    if (is.null(hazard))
    {
        return(
            matrix(
                NA,
                nrow = length(elevation),
                ncol = length(ordered_parameter),
                dimnames = list("asset" = asset, "hazard" = ordered_parameter)
            )
        )
    } else {
        if (is.vector(hazard)) hazard = t(hazard)
        if (!all(ordered_parameter %in% colnames(hazard)))
            stop("hazard is not well formatted.")
        if (nrow(hazard) == 1)
            hazard = matrix(
                hazard[,ordered_parameter],
                byrow = TRUE,
                nrow = length(elevation),
                ncol = length(ordered_parameter),
                dimnames = list("asset" = asset, "hazard" = ordered_parameter)
            )
        if (is.na(b_elevation)) b_elevation = 0
        elevation = elevation + b_elevation
        hazard[ , "height"] = hazard[ , "height"] - elevation
        return(hazard[ , ordered_parameter])
    }
}