# AVA <img src="man/figures/logo.png" align="right" width="25%" height="25%"/>

## Welcome to AVA

**AVA**

`AVA` is an implementation in R language of the model developed in [Pauline Brémond's thesis](https://www.theses.fr/2011MON10059) for producing indicators of farm vulnerability to floodings. 
`AVA` stands for Agricultural Vulnerability Assessment ("Évaluation de la Vulnérabilité Agricole" in French).

This model was developed as part of a project called EVA under the Rhône River Program. It received funding from the FEDER and the Inter-Regional Project Contract. The results of the application of the model as part of the funded project are available in [the report Blanc et al (2010)](https://www.plan-rhone.fr/fileadmin/medias/Publications/Inondations/Reviter-guide/Compl/2010_CEMAGREF-PlanRhone_Evaluation-Economique-Vulnerabilite-Agricole.pdf)

## What **AVA** can do

**AVA** evaluates flood impacts on farms with quantitative indicators in order to assess their vulnerability taking into account farm recovery after flooding. AVA includes not only direct consequences but also induced consequences related to farm activity perturbations after flooding.

**AVA** represents the farm as a collection of physical components determined by their usability: (i) the buildings containing machinery and stocks of inputs; (ii) the parcels of land containing crop production and plant material (such as vineyards). It also considers farmer's decision making without flood based on the crop management sequence. It also considers the possibility for farmers to have access to external resources during recovery.

The indicators that can be produced using **AVA** are: 

- the chronology of the usability of every farm physical component; 
- a quantification of changes induced in needs and availability of production factors, mainly workforce and machinery, required to carry on, at the same time, both production and recovery tasks;
- monetary damage distinguished in:
    - gross damage which corresponds to the total amount of damage before the farmer received insurance payment, 
    - net damage which is the total amount of damage after insurance payment, 
    - direct damage which corresponds to the physical damage on farm components (building, machinery, crop, soil and plant material),
    - induced damage which corresponds to the loss of activity or the extra charges during the recovery period after flooding.

## What **AVA** cannot do

**AVA** cannot (and is not intended to):

- calculate damage from the intersection of hydraulic modeling, land use modeling
- calculate damage resulting from interactions between farmers or with their economic partners (cooper performs this aspect)

**AVA** cannot (but should at some point be able to) :

- calculate precisely damage to a diversity of crop ([floodam.agri](http://www.floodam.org/floodam.agri) performs better this part)
- calculate precisely damage to a diversity of farm's buidling ([floodam.building](http://www.floodam.org/floodam.building) performs better this part)

# How to get **AVA**?

You can download and install it from this archive: [www.floodam.org/library/ava_0.9.0.0.tar.gz](http://www.floodam.org/library/ava_0.9.0.0.tar.gz).


You can download and install it from this archive:
[www.floodam.org/library/ava_0.9.0.0.tar.gz]
(http://www.floodam.org/library/ava_0.9.0.0.tar.gz).

For instance, in a linux environment, you can use those bash instructions:

```
# without devtools
wget www.floodam.org/library/ava_0.9.0.0.tar.gz
R -e "install.packages(pkgs = 'ava_0.9.0.0.tar.gz', type = 'source', repos = NULL, dependencies = TRUE)"
rm ava_0.9.0.0.tar.gz

# with devtools
R -e "devtools::install_url('www.floodam.org/library/ava_0.9.0.0.tar.gz')"
```

In case you are not using a linux environment, you can do the same within a R session:

```
# without devtools
archive = "ava_0.9.0.0.tar.gz"
url = file.path("www.floodam.org/library", archive)
download.file(url = url, destfile = archive)
install.packages(pkgs = archive, type = 'source', repos = NULL, dependencies = TRUE)
unlink(archive)

# with devtools
devtools::install_url('www.floodam.org/library/ava_0.9.0.0.tar.gz')
```

## How to help for the development of **AVA**?

Contact us.
